# Copyright (c) 2025 Marcin Zdun
# This code is licensed under MIT license (see LICENSE for details)

"""
The **cxx_flow.__main__** allows *C++ flow* to be called as Python component.
"""

from .flow.cli import main

main()
