# Copyright (c) 2025 Marcin Zdun
# This code is licensed under MIT license (see LICENSE for details)

"""
The **cxx_flow.plugins** provides the runtime comnponents of the *C++ flow*:
the :class:`initialization helpers <cxx_flow.api.init.InitStep>` and
:class:`run steps <cxx_flow.api.step.Step>`.
"""
